@@ -61,15 +61,12 @@ using namespace Nepomuk2::Vocabulary;
 
 Nepomuk2::ResourceData::ResourceData( const QUrl& uri, const QUrl& kickOffUri, const QUrl& type, ResourceManagerPrivate* rm )
     : m_uri(uri),
-      m_type(type),
+      m_type(type.isEmpty() ? RDFS::Resource() : type),
       m_modificationMutex(QMutex::Recursive),
       m_cacheDirty(false),
       m_addedToWatcher(false),
       m_rm(rm)
 {
-    if( type.isEmpty() )
-        m_type = RDFS::Resource();
-
     m_rm->dataCnt.ref();
 
     if( !uri.isEmpty() ) {
@@ -179,19 +176,20 @@ void Nepomuk2::ResourceData::resetAll( bool isDelete )
             }
 
             // remove this Resource from the list of watched resources
-            QMetaObject::invokeMethod(m_rm->m_watcher, "removeResource", Qt::AutoConnection, Q_ARG(Nepomuk2::Resource, Resource::fromResourceUri(m_uri)));
+            QMetaObject::invokeMethod(m_rm->m_watcher, "removeResource", Qt::AutoConnection, Q_ARG(QUrl, m_uri));
             m_addedToWatcher = false;
         }
     }
     m_rm->mutex.unlock();
 
     // reset all variables
+    QMutexLocker locker(&m_modificationMutex);
     m_uri.clear();
     m_nieUrl.clear();
     m_naoIdentifier.clear();
     m_cache.clear();
     m_cacheDirty = false;
-    m_type.clear();
+    m_type = RDFS::Resource();
 }
 
 
@@ -304,7 +302,7 @@ bool Nepomuk2::ResourceData::store()
     return true;
 }
 
-
+// Caller must hold m_modificationMutex
 void Nepomuk2::ResourceData::addToWatcher()
 {
     if(!m_rm->m_watcher) {
@@ -320,10 +318,10 @@ void Nepomuk2::ResourceData::addToWatcher()
                             m_rm->m_manager, SLOT(slotPropertyAdded(Nepomuk2::Resource, Nepomuk2::Types::Property, QVariant)) );
         QObject::connect( m_rm->m_watcher, SIGNAL(propertyRemoved(Nepomuk2::Resource, Nepomuk2::Types::Property, QVariant)),
                             m_rm->m_manager, SLOT(slotPropertyRemoved(Nepomuk2::Resource, Nepomuk2::Types::Property, QVariant)) );
-        m_rm->m_watcher->addResource( Nepomuk2::Resource::fromResourceUri(m_uri) );
+        m_rm->m_watcher->addResource( m_uri );
     }
     else {
-        QMetaObject::invokeMethod(m_rm->m_watcher, "addResource", Qt::AutoConnection, Q_ARG(Nepomuk2::Resource, Nepomuk2::Resource::fromResourceUri(m_uri)) );
+        QMetaObject::invokeMethod(m_rm->m_watcher, "addResource", Qt::AutoConnection, Q_ARG(QUrl, m_uri) );
     }
     // (re-)start the watcher in case this resource is the only one in the list of watched
     if(m_rm->m_watcher->resources().count() <= 1) {
@@ -334,6 +332,7 @@ void Nepomuk2::ResourceData::addToWatcher()
 
 bool Nepomuk2::ResourceData::load()
 {
+    QMutexLocker rmlock(&m_rm->mutex); // for updateKickOffLists, but must be locked first
     QMutexLocker lock(&m_modificationMutex);
 
     if ( m_cacheDirty ) {
@@ -353,6 +352,7 @@ bool Nepomuk2::ResourceData::load()
                 Soprano::Node o = it["o"];
                 Nepomuk2::Variant var = Variant::fromNode( o );
                 //FIXME: Is this updating of the kickoff lists required?
+                // (note: remove 'rmlock'if you remove this)
                 updateKickOffLists( p, var );
                 m_cache[p].append( var );
             }
@@ -376,15 +376,19 @@ void Nepomuk2::ResourceData::setProperty( const QUrl& uri, const Nepomuk2::Varia
 
     if( store() ) {
         // step 0: make sure this resource is in the store
+        QMutexLocker rmlock(&m_rm->mutex); // for updateKickOffLists, but must be locked first
         QMutexLocker lock(&m_modificationMutex);
 
         // update the store
         QVariantList varList;
-        foreach( const Nepomuk2::Variant var, value.toVariantList() ) {
-            // make sure resource values are in the store
+        foreach( const Variant& var, value.toVariantList() ) {
+            // make sure resource values are identified and in the store
             if( var.simpleType() == qMetaTypeId<Resource>() ) {
-                var.toResource().m_data->store();
-                varList << var.toUrl();
+                Resource res = var.toResource();
+                res.determineFinalResourceData();
+                res.m_data->store();
+
+                varList << res.uri();
             }
             else {
                 varList << var.variant();
@@ -417,6 +421,7 @@ void Nepomuk2::ResourceData::addProperty( const QUrl& uri, const Nepomuk2::Varia
 
     if( value.isValid() && store() ) {
         // step 0: make sure this resource is in the store
+        QMutexLocker rmlock(&m_rm->mutex); // for updateKickOffLists, but must be locked first
         QMutexLocker lock(&m_modificationMutex);
 
         // update the store
@@ -453,9 +458,10 @@ void Nepomuk2::ResourceData::addProperty( const QUrl& uri, const Nepomuk2::Varia
 void Nepomuk2::ResourceData::removeProperty( const QUrl& uri )
 {
     Q_ASSERT( uri.isValid() );
-    if( !m_uri.isEmpty() ) {
-        QMutexLocker lock(&m_modificationMutex);
+    QMutexLocker rmlock(&m_rm->mutex); // for updateKickOffLists, but must be locked first
+    QMutexLocker lock(&m_modificationMutex);
 
+    if( !m_uri.isEmpty() ) {
         KJob* job = Nepomuk2::removeProperties(QList<QUrl>() << m_uri, QList<QUrl>() << uri);
         if( !job->exec() ) {
             //TODO: Set the error somehow
@@ -509,9 +515,11 @@ bool Nepomuk2::ResourceData::isValid() const
     return !m_uri.isEmpty() || !m_nieUrl.isEmpty() || !m_naoIdentifier.isEmpty();
 }
 
-
+// Caller must hold the ResourceManager mutex (since the RM owns the returned ResourceData pointer)
 Nepomuk2::ResourceData* Nepomuk2::ResourceData::determineUri()
 {
+    QMutexLocker lock(&m_modificationMutex);
+
     // We have the following possible situations:
     // 1. m_uri is already valid
     //    -> simple, nothing to do
@@ -636,7 +644,6 @@ void Nepomuk2::ResourceData::updateUrlLists(const QUrl& newUrl)
 {
     const QUrl oldUrl = m_cache[NIE::url()].toUrl();
 
-    QMutexLocker rmlock(&m_rm->mutex);
     m_rm->m_urlKickOff.remove( oldUrl );
 
     if( !newUrl.isEmpty() ) {
@@ -648,7 +655,6 @@ void Nepomuk2::ResourceData::updateIdentifierLists(const QString& newIdentifier)
 {
     const QString oldIdentifier = m_cache[NAO::identifier()].toString();
 
-    QMutexLocker rmlock(&m_rm->mutex);
     m_rm->m_identifierKickOff.remove( oldIdentifier );
 
     if( !newIdentifier.isEmpty() ) {
@@ -656,6 +662,7 @@ void Nepomuk2::ResourceData::updateIdentifierLists(const QString& newIdentifier)
     }
 }
 
+// Caller must lock RM mutex  first
 void Nepomuk2::ResourceData::updateKickOffLists(const QUrl& uri, const Nepomuk2::Variant& variant)
 {
     if( uri == NIE::url() )
@@ -663,3 +670,36 @@ void Nepomuk2::ResourceData::updateKickOffLists(const QUrl& uri, const Nepomuk2:
     else if( uri == NAO::identifier() )
         updateIdentifierLists( variant.toString() );
 }
+
+void Nepomuk2::ResourceData::propertyRemoved( const Types::Property &prop, const QVariant &value_ )
+{
+    QMutexLocker lock(&m_modificationMutex);
+    QHash<QUrl, Variant>::iterator cacheIt = m_cache.find(prop.uri());
+    if(cacheIt != m_cache.end()) {
+        Variant v = *cacheIt;
+        const Variant value(value_);
+        QList<Variant> vl = v.toVariantList();
+        if(vl.contains(value)) {
+            vl.removeAll(value);
+            if(vl.isEmpty()) {
+                updateKickOffLists(prop.uri(), Variant());
+                m_cache.erase(cacheIt);
+            }
+            else {
+                // The kickoff properties (nao:identifier and nie:url) both have a cardinality of 1
+                // If we have more than one value, then the properties must not be any of them
+                if( vl.size() == 1 )
+                    updateKickOffLists(prop.uri(), vl.first());
+                cacheIt.value() = vl;
+            }
+        }
+    }
+}
+
+void Nepomuk2::ResourceData::propertyAdded( const Types::Property &prop, const QVariant &value )
+{
+    QMutexLocker lock(&m_modificationMutex);
+    const Variant var(value);
+    updateKickOffLists(prop.uri(), var);
+    m_cache[prop.uri()].append(var);
+}
